	   /****************************************************/
	   /* Program pro analyzu zdrojoveho programu jazyka C */
	   /*    Program informuje o promennych ktere byly     */
	   /*    deklarovany, avsak nikde nebyly pouzity .     */
	   /****************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>

/* maximalni delka souboru */
#define MAX 32767
/* pocet klicovych slov ulozenych v 'datatype' */
#define WORD 10

static char source[MAX];                     /* pole pro ulozeni souboru */
int         error = 0,                   /* pocet nepouzitych promennych */
	    len   = 0;                              /* delka pole source */
static char *datatype[] = {     "#define",
				"void",
				"int",
				"short",
				"long",                /* klicova slova */
				"unsigned",
				"float",
				"double",
				"char",
				"FILE"        };


void readdata(char *filename)      /* funkce pro nacteni souboru do pole */
{
	int  c,                    /* ascii hodnoty znaku ze vstupu */
	     d,
	     k,                    /* pomocna promenna */
	     in_rem = 0,           /* indikace ze jsme uvnitr komentare */
	     in_zav = 0,           /* indikace ze jsme uvnitr zavorek   */
	     in_uvoz = 0;          /* indikace ze jsme uvnitr uvozovek  */

	FILE *fp, *fopen();            /* otevreni souboru */
	fp = fopen(filename, "r");

	c = getc(fp);
	while( c != EOF ) {            /* nacitani hodnot ze vstupu */
		d = getc(fp);

		switch( c )  {         /* Pokud se nachazime uvnitr */
		case '(':              /* komentare, uvozovek       */
			++in_zav;      /* tak misto nich se budou   */
			break;         /* ukladat mezery          . */
		case ')':
			--in_zav;
			break;
		case '"':
			k = ( !in_uvoz ) ? 1 : -1;
			in_uvoz += k;
			in_rem += k;
			break;
		case '/':
			if( d == '*' ) {
				d = ' ';
				++in_rem;
				}
			break;
		case '*':
			if( d == '/' ) {
				--in_rem;
				c = ' ';
				d = ' ';
				}
			break;
		}                          /* Pokud jsme uvnitr zavorek  */
					   /* vsecny carky jsou zameneny */
		if( in_rem > 0 )           /* na stredniky    (funkce) . */
			c = ' ';
		else
			if( in_zav > 0 && c == ',' )
				c = ';';

		source[len++] = c;          /* ulozeni znaku do pole     */
		c = d;                      /* prirazeni pro test na EOF */
	}
	source[len] = '\0';                 /* ukonceni pole */

	fclose(fp);                         /* zavreni souboru */

return;
}

				     /* Funkce vyhledavajici retezec     */
int strnsrch(char *substr, int n)    /* 'substr' v poli 'source'         */
{                                    /* od 'n'-teho indexu pole 'source'.*/
	int x,                       /* index v podretezci 'substr' */
	    l;                       /* delka 'substr' */

	l = strlen(substr);

	while( n < len - l + 1 ) {
		for(x = 0;; ++x) {
			if( source[n + x] != substr[x] )
				break;
			if( x == l - 1 ) {
/* Test na znak pred */		if( n > 0 && isalnum(source[n - 1]) )
/* a za nalezenym    */					break;
/* podretezcem     . */		if( n != len - l && isalnum(source[n + l]) )
/* Vraci hodnotu za  */			break;
/* poslednim znakem. */		return(n + l + 1);
				}
		}
		++n;
	}
return(-1);                     /* navratova hodnota pokud nenalezne */
}
				/* Funkce zjistuje jmeno promenne */
				/* za nalezenym klicovym slovem   */
int datascan(int n)             /* a testuje jestlize je pouzito. */
{
	int  x,                 /* index v podretezci 'varname' */
	     m,                 /* pomocna promenna */
	     f;                 /* index pro volani funkce 'strnsrch' */
	char varname[256];      /* pole pro ulozeni nazvu promenne */

	for(;;) {
					      /* Zvysuje 'n' (= 'pozice') */
		while( !isalpha(source[n]) )  /* dokud nenarazi na        */
			++n;                  /* znak abecedy       .     */

		x = 0;
		do {                                    /* Nacteni jmena */
			varname[x++] = source[n++];     /* promenne    . */
		} while( isalnum(source[n]) );
		varname[x] = '\0';

		if( !strcmp(varname, "main") )        /* "main" muze byt */
			return(n);                    /* obsazen jednou. */
		for(m = 0; m < WORD; ++m)
			if( !strcmp(varname, datatype[m]) )
				break;
		if( m != WORD )               /* Test na declarace typu  */
			continue;             /* napr. long int varname. */

				   /* testovani duplicity jmena promenne */
		f = n;
		if( strnsrch(varname, f) < 0 )  {
		  printf("\nPromenna '%s' nebude nikdy pouzita ?!", varname);
		  ++error;
		  }

		for(;;) {                     /* Test na dalsi pripadne  */
			m = source[n++];      /* jmeno promenne stejneho */
			if( m == ',' )        /* typu za carkou        . */
				break;
			if( m == ';' || m == '(' || m == ')' )
				return(n);
			}
		}
}


int main(int argc, char **argv)
{
	int tindex,                /* index v 'datatype */
	    pozice;                /* pozice indexu v poli source */

/* Napoveda */
	if( argc == 1 ) {
		printf("\nHELP :      declare.exe -h\n");
		return(0);
		}

 if( !strcmp(argv[1], "-h") ) {
  printf("\n              Program pro analyzu zdrojoveho programu jazyka C.");
  printf("\n 	          Program informuje o promennych ktere byly");
  printf("\n	          deklarovany, avsak nikde nebyly pouzity .");
  printf("\n                     RUN :        declare.exe <FILENAME>\n");
  return(0);
  }

	readdata(argv[1]);   /* volani nacitaci funkce, arg[1] = FILENAME */

	if( !len ) {                                    /* file not found */
		printf("\nSoubor nebyl nalezen !\n");
		return(0);
		}
						     /*  Nacitani        */
	for(tindex = 0; tindex < WORD; ++tindex) {   /*  klicovych slov. */
		for(pozice = 0;;) {
			pozice = strnsrch(datatype[tindex], pozice);
			if( pozice < 0 )
				break;       /* test nalezeni klic. slov */
			else
				pozice = datascan(pozice);
			}
	}			   /* Pokud nasel vola funkci 'datascan' */
				   /* 'pozice' = 2 znaky za klic. slovem */


	printf("\n");              /* testuje nalezeni chyby */
	if( !error )
		printf("Zdrojovy text je v poradku !\n");

return(0);
}